from typing import Optional, List

from fastapi_users.db import SQLAlchemyBaseUserTable
from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, relationship, mapped_column

from backend.database import Base, intpk


class UserRoleAssociation(Base):
    """
    Модель таблицы связи Many2Many для Role & User
    """
    __tablename__ = "user_role_association"
    user_id: Mapped[int] = mapped_column(ForeignKey("user.id"), primary_key=True)
    role_id: Mapped[int] = mapped_column(
        ForeignKey("role.id"), primary_key=True
    )
    extra_data: Mapped[Optional[str]]
    child: Mapped["Role"] = relationship()

    
class Role(Base):
    """
    Модель таблицы role
    """
    __tablename__ = "role"

    id: Mapped[intpk]
    role: Mapped[str]
    

class User(SQLAlchemyBaseUserTable[int], Base):
    """
    Модель таблицы user
    """
    __tablename__ = "user"
    id: Mapped[intpk]
    children: Mapped[List["UserRoleAssociation"]] = relationship()
    
